#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#		Name:	Stitch Helix
#	 Usage:	Create TIN between two Follow Me Helix curves created with different radii.
#		Note: The two helixes must have the same linear and angular increments so that
# 	Date:	Sep 2011
#------------------------------------------------------------------------------------------------

require 'sketchup'

class StitchIt

	def initialize
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@best = nil
		@edge = nil
		@aMod = Sketchup.active_model
		@aEnt = @aMod.active_entities
		@aSel = @aMod.selection
		self.reset
	end

	def reset
		@state = 0
		@ip1.clear
		@aSel.clear
		@drawn = false
	end
	
	def onMouseMove(flags, x, y, view)
		self.set_current_point(x, y, view)
		ph = view.pick_helper
		ph.do_pick(x, y)
		@edge = ph.picked_edge if ph.picked_edge != @edge
		@best = ph.best_picked if ph.best_picked != @best
		Sketchup::set_status_text("Select First Helix", SB_PROMPT) if @state == 0
		Sketchup::set_status_text("Select Second Helix", SB_PROMPT) if @state == 1
		view.invalidate
	end

	def set_current_point(x, y, view)
		if( !@ip.pick(view, x, y, @ip1) )
			return false
		end
		need_draw = true
		view.tooltip = @ip.tooltip     
		need_draw = @ip.display? || @drawn
		view.invalidate if need_draw
	end

	def onLButtonDown(flags, x, y, view)
		self.set_current_point(x, y, view)
		if (@edge && @best.is_a?(Sketchup::Edge)) then
			if @state == 0
				@aMod.start_operation "Stitch It"
				if @edge.curve
					edges1=@edge.curve.edges
					@pts1=@edge.curve.vertices
					@pts1.collect!{|v| v.position}
					@aSel.clear; @aSel.add edges1
					@state += 1
				else
					UI.messagebox "Not a curve"
				end
			elsif @state == 1
				if @edge.curve
					edges2=@edge.curve.edges
					@pts2=@edge.curve.vertices
					@pts2.collect!{|v| v.position}
					@aSel.add edges2
					if UI.messagebox("Continue?",MB_YESNO) == 6
						for i in 1..@pts1.length-1
							f=@aEnt.add_face(@pts1[i-1],@pts2[i-1],@pts1[i])
							f.edges.each {|x| x.soft = true if x.faces.length>1 };f.material=[142,142,142]
							f=@aEnt.add_face(@pts2[i-1],@pts2[i],@pts1[i])
							f.edges.each {|x| x.soft = true if x.faces.length>1};f.material=[142,142,142]
						end
						@aMod.commit_operation
						onCancel(nil,nil)
					else
						self.reset
					end
				else
					UI.messagebox "Not a curve"
				end
			end
		else
			UI.beep
		end
	end
	
	def onCancel(flag, view)
		Sketchup.send_action "selectSelectionTool:"
	end
	
	def onRButtonDown(flags, x, y, view)
		Sketchup.send_action "selectSelectionTool:" if @state == 0		
		self.reset
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? ) then
			@ip.draw(view)
			@drawn = true
		end
		if @edge
			view.drawing_color = "blue"
			view.line_width = 3
			view.draw_line @edge.start.position,@edge.end.position
		end
	end
end # of class StitchIt
#----------------------------------------------------------------------------
if( not file_loaded?("Stitch Helix.rb") )
    UI.menu("Plugins").add_item("Stitch Helix") { Sketchup.active_model.select_tool StitchIt.new }
end
#-----------------------------------------------------------------------------
file_loaded("Stitch Helix.rb")

